@@ -3,6 +3,7 @@ package tplimpl_test
 import (
 	"context"
 	"io"
+	"strings"
 	"testing"
 
 	qt "github.com/frankban/quicktest"
@@ -684,6 +685,102 @@ layout: mylayout
 	b.AssertFileContent("public/en/foo/index.xml", "layouts/list.xml")
 }
 
+func TestLookupOrderIssue13636(t *testing.T) {
+	t.Parallel()
+
+	filesTemplate := `
+-- hugo.toml --
+defaultContentLanguage = "en"
+defaultContentLanguageInSubdir = true
+[languages]
+[languages.en]
+weight = 1
+[languages.nn]
+weight = 2
+-- content/s1/p1.en.md --
+---
+outputs: ["html", "amp", "json"]
+---
+-- content/s1/p1.nn.md --
+---
+outputs: ["html", "amp", "json"]
+---
+-- layouts/L1 --
+L1
+-- layouts/L2 --
+L2
+-- layouts/L3 --
+L3
+
+`
+
+	tests := []struct {
+		Lang       string
+		L1         string
+		L2         string
+		L3         string
+		ExpectHTML string
+		ExpectAmp  string
+		ExpectJSON string
+	}{
+		{"en", "all.en.html", "all.html", "single.html", "single.html", "single.html", ""},
+		{"en", "all.amp.html", "all.html", "page.html", "page.html", "all.amp.html", ""},
+		{"en", "all.amp.html", "all.html", "list.html", "all.html", "all.amp.html", ""},
+		{"en", "all.en.html", "all.json", "single.html", "single.html", "single.html", "all.json"},
+		{"en", "all.en.html", "single.json", "single.html", "single.html", "single.html", "single.json"},
+		{"en", "all.en.html", "all.html", "list.html", "all.en.html", "all.en.html", ""},
+		{"en", "list.en.html", "list.html", "list.en.html", "", "", ""},
+		{"nn", "all.en.html", "all.html", "single.html", "single.html", "single.html", ""},
+		{"nn", "all.en.html", "all.nn.html", "single.html", "single.html", "single.html", ""},
+		{"nn", "all.en.html", "all.nn.html", "single.nn.html", "single.nn.html", "single.nn.html", ""},
+		{"nn", "single.json", "single.nn.json", "all.json", "", "", "single.nn.json"},
+		{"nn", "single.json", "single.en.json", "all.nn.json", "", "", "single.json"},
+	}
+
+	for i, test := range tests {
+		if i != 8 {
+			// continue
+		}
+		files := strings.ReplaceAll(filesTemplate, "L1", test.L1)
+		files = strings.ReplaceAll(files, "L2", test.L2)
+		files = strings.ReplaceAll(files, "L3", test.L3)
+		t.Logf("Test %d: %s %s %s %s", i, test.Lang, test.L1, test.L2, test.L3)
+
+		for range 3 {
+			b := hugolib.Test(t, files)
+			b.Assert(len(b.H.Sites), qt.Equals, 2)
+
+			var (
+				pubhHTML = "public/LANG/s1/p1/index.html"
+				pubhAmp  = "public/LANG/amp/s1/p1/index.html"
+				pubhJSON = "public/LANG/s1/p1/index.json"
+			)
+
+			pubhHTML = strings.ReplaceAll(pubhHTML, "LANG", test.Lang)
+			pubhAmp = strings.ReplaceAll(pubhAmp, "LANG", test.Lang)
+			pubhJSON = strings.ReplaceAll(pubhJSON, "LANG", test.Lang)
+
+			if test.ExpectHTML != "" {
+				b.AssertFileContent(pubhHTML, test.ExpectHTML)
+			} else {
+				b.AssertFileExists(pubhHTML, false)
+			}
+
+			if test.ExpectAmp != "" {
+				b.AssertFileContent(pubhAmp, test.ExpectAmp)
+			} else {
+				b.AssertFileExists(pubhAmp, false)
+			}
+
+			if test.ExpectJSON != "" {
+				b.AssertFileContent(pubhJSON, test.ExpectJSON)
+			} else {
+				b.AssertFileExists(pubhJSON, false)
+			}
+		}
+	}
+}
+
 func TestLookupShortcodeDepth(t *testing.T) {
 	t.Parallel()
 
@@ -941,6 +1038,35 @@ _markup/render-codeblock-goat.html
 	b.AssertFileContent("public/index.html", "_markup/render-codeblock.html_markup/render-codeblock-goat.html")
 }
 
+func TestLookupCodeblockIssue13651(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+-- layouts/all.html --
+{{ .Content }}|
+-- layouts/_markup/render-codeblock-foo.html --
+render-codeblock-foo.html
+-- content/_index.md --
+---
+---
+
+§§§
+printf "Hello, world!"
+§§§
+
+§§§ foo
+printf "Hello, world again!"
+§§§
+`
+
+	b := hugolib.Test(t, files)
+
+	content := b.FileContent("public/index.html")
+	fooCount := strings.Count(content, "render-codeblock-foo.html")
+	b.Assert(fooCount, qt.Equals, 1)
+}
+
 // Issue #13515
 func TestPrintPathWarningOnDotRemoval(t *testing.T) {
 	t.Parallel()
@@ -959,7 +1085,7 @@ All.
 
 	b := hugolib.Test(t, files, hugolib.TestOptWarn())
 
-	b.AssertLogContains("Duplicate content path")
+	b.AssertLogContains("! Duplicate content path")
 }
 
 // Issue #13577.
@@ -1204,3 +1330,69 @@ layouts/_partials/comment.ru.xml
 	b.AssertFileContent("public/ru/index.html", "layouts/_partials/comment.ru.html") // fail
 	b.AssertFileContent("public/ru/index.xml", "layouts/_partials/comment.ru.xml")   // fail
 }
+
+func TestLayoutIssue13628(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableKinds = ['home','rss','sitemap','taxonomy','term']
+-- content/p1.md --
+---
+title: p1
+layout: foo
+---
+-- layouts/single.html --
+layouts/single.html
+-- layouts/list.html --
+layouts/list.html
+`
+
+	for range 5 {
+
+		b := hugolib.Test(t, files)
+
+		b.AssertFileContent("public/p1/index.html", "layouts/single.html")
+	}
+}
+
+func TestTemplateLoop(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+-- layouts/_partials/p.html --
+p: {{ partial "p.html" . }}
+-- layouts/all.html --
+{{ partial "p.html" . }}
+
+`
+	b, err := hugolib.TestE(t, files)
+	b.Assert(err, qt.IsNotNil)
+	b.Assert(err.Error(), qt.Contains, "error calling partial: maximum template call stack size exceeded")
+}
+
+func TestIssue13630(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableKinds = ['rss','sitemap']
+-- content/p1.md --
+---
+title: p1
+layout: foo
+---
+-- layouts/list.html --
+layouts/list.html
+-- layouts/taxononmy.html.html --
+layouts/taxononmy.html.html
+`
+
+	var b *hugolib.IntegrationTestBuilder
+
+	for range 3 {
+		b = hugolib.Test(t, files)
+		b.AssertFileExists("public/p1/index.html", false)
+	}
+}
